# -*- coding: UTF-8 -*-
#
# Bacula Python interface script for the Director
#

import sys, commands, os, time, pickle
import bacula
#from bbackup import backup
sys.path.append('/usr/share/eole/bacula/')
from baculasupport import typeSupport
from creole import parsedico

class BaculaEvents(object):
    def __init__(self):
        self.repertoire = '/var/sauvegardes'
        self.support = typeSupport().liste()
        self.service = '/usr/share/eole/bacula/baculaservices.sh'
        self.eadfile = '/var/www/ead/tmp/resultat-bacula'
        self.eadrapport = '/var/www/ead/tmp/rapport-bacula.txt'
        self.msgfile = '/etc/bacula/eolemsgdefs.pic'

    def JobStart(self, job):
        self.support = typeSupport().liste()
        self.job = job
        # Effacement du rapport de la sauvegarde précédente
        file(self.eadrapport, 'w').write('')
        os.chmod(self.eadrapport, 0644)

        # s'il s'agit d'une sauvegarde
        if self.job.Type == 'Backup':
            self.job.JobReport = "Démarrage d'une sauvegarde type %s\n"%self.support
            if self.support in ['distant', 'usb']:
                if not self.montage():
                    self.genmessage(False)
                    self.mailadmin()
                    return
#                if not self.comparatif():
#                    self.genmessage(False)
#                    self.mailadmin()
#                    return
        
            # commandes perso
            if os.path.isfile('/etc/bacula/precommandes'):
                self.lance_cmd('sh /etc/bacula/precommandes')
        
            # préparation de la sauvegarde (SID etc.)
            self.lance_cmd('%s prebackup'%self.service)
            # arrêt des services
            self.lance_cmd('%s servicestop'%self.service)

        # restauration:
        elif self.job.Type == 'Restore':
            # arrêt des services
            self.lance_cmd('%s servicestop'%self.service)

    # Bacula Job is going to terminate
    def JobEnd(self, job):
        self.job = job
        # s'il s'agit d'une sauvegarde
        if self.job.Type == 'Backup':
            # commande post-backup Eole
            self.lance_cmd('%s postbackup'%self.service)
            # commandes perso
            if os.path.isfile('/etc/bacula/postcommandes'):
                self.lance_cmd('sh /etc/bacula/postcommandes')
            # redémarrage des services
            self.lance_cmd('%s servicestart'%self.service)
            if self.support in ['distant', 'usb']:
                #démontage de self.repertoire
                self.demontage()
            # génération du message de l'EAD
            if job.JobStatus == 'T':
                self.genmessage(True)
            else:
                self.genmessage(False)
                self.mailadmin()

        # restauration:
        elif self.job.Type == 'Restore':
            # redémarrage des services
            self.lance_cmd('%s servicestart'%self.service)
            self.lance_cmd('%s postrestore'%self.service)
            self.demontage()
            message = """
################################
# La restauration est terminée #
#    Appuyez sur Ctrl+C et     #
#   Lancez 'reconfigure' !!!   #
################################
"""
            self.job.JobReport = message
            
        job.JobReport="Python Dir JobEnd output: JobId=%d Status=%s Client=%s Type=%s.\n" % (job.JobId, job.JobStatus, job.Client, job.Type) 
        
    # Called here when the Bacula daemon is going to exit
    def Exit(self, job):
        print  "Daemon exiting."

    #fonctions Eole
    def lance_cmd(self, cmd):
        """lance la commande cmd, récupère la sortie standard
        et l'envoie dans les logs Bacula
        """
        try:
            rcode = commands.getoutput(cmd)
            self.job.JobReport = '%s\n'%rcode
        except:
            # si la commande ne renvoie rien
            os.system(cmd)
        return True
        
    def montage(self):
        """monte le support self.support sur $REPERTOIRE
        défini dans self.service
        """
        rcode = commands.getoutput('%s %s'%(self.service, self.support))
        if rcode.count('OK') == 0:
            self.job.JobReport = '%s\n'%rcode
            self.job.JobReport = "ABANDON !!!\n"
            self.job.cancel(self.job.JobId)
            return False
        else:
            self.job.JobReport = '%s\n'%rcode
            return True
    
    def demontage(self):
        """démonte self.repertoire
        qui doit être identique à $REPERTOIRE dans self.service
        """
        os.system('/bin/umount %s'%self.repertoire)
    
    def estime_sav(self):
        """récupère l'estimation faite par Bacula
        et la renvoie en Ko
        """
        mavar = commands.getoutput('/bin/echo -e "estimate level=%s\n1"|/usr/bin/bconsole -c /etc/bacula/bconsole.conf'%self.job.Level).split('bytes=')
        
        estimation = long(mavar[1].split('\n')[0].replace(',',''))
        #en kilo Octects
        kestimation = estimation/1024
        return kestimation
    
    def comparatif(self):
        """Vérifie qu'il y a suffisamment d'espace libre pour effectuer la sauvegarde
        en ajoutant 10Mo de marge.
        """
        espacelibre = long(commands.getoutput("/bin/df -kP | /bin/grep %s | awk '{print $4}'"%self.repertoire))
        #10 Mo de marge
        espacelibre = espacelibre - 10240
        estimation = self.estime_sav()
        if espacelibre < 0:
            espacelibre = 0
        if espacelibre < estimation:
            self.job.JobReport = "#########################################################################\n"
            self.job.JobReport = "# Espace insuffisant\n"
            self.job.JobReport = "# Espace libre restant : %i Ko. (+ 10 Mo de marge)\n"%espacelibre
            self.job.JobReport = "# Espace nécessaire : %i Ko.\n"%estimation
            self.job.JobReport = "#########################################################################\n"
            self.job.JobReport = "ABANDON !!!\n"
            self.job.JobReport = "\n"
            self.demontage()
            self.job.cancel(self.job.JobId)
            return False
        else:
            self.job.JobReport = "Espace libre OK\n"
            return True

    def genmessage(self, resultat):
        """Génère le message affiché dans l'EAD en fonction du résultat de la sauvegarde
        """
        dicojour = { 'mon':'lundi', 'tue':'mardi', 'wed':'mercredi', 'thu':'jeudi', 'fri':'vendredi', 'sat':'samedi', 'sun':'dimanche', 'Incremental':'incrémentale', 'Full':'complète' }
        dicomois = {'jan':'janvier', 'feb':'février', 'mar':'mars', 'apr':'avril', 'may':'mai', 'jun':'juin', 'jul':'juillet', 'aug':'août', 'sep':'septembre', 'oct':'octobre', 'nov':'novembre', 'dec':'décembre'}
        
        jour, mois, numjour, heure, annee = time.asctime().lower().split()
        jour = dicojour[jour]
        mois = dicomois[mois]
        
        if resultat:
            message = 'Dernière sauvegarde effectuée le %s %s %s %s à %s.\n'%(jour, numjour, mois, annee, heure)
        else:
            message = 'La sauvegarde du %s %s %s %s à %s, a échoué.\n'%(jour, numjour, mois, annee, heure)
        file(self.eadfile, 'w').write(message)
        os.chmod(self.eadfile, 0644)
        return 0

    def mailadmin(self):
        """envoie un mail à l'adresse renseignée dans l'EAD (/etc/bacula/eolemsgdefs.pic) => self.msgfile
        pour prévenir d'un incident pendant la sauvegarde.
        Le système de mail Bacula n'envoie pas de mail si l'erreur a lieu dans JobStart()
        """
        try:
            dico = pickle.load(file(self.msgfile,'r'))
            dicoeole = parsedico.parse_dico()
            
            dico['nom_domaine'] = dicoeole['nom_domaine_local']
            dico['numero_etab'] = dicoeole['numero_etab']
            dico['message'] = """\"La sauvegarde dans l'établissement %(nom_domaine)s a échoué.\nVeuillez consulter le journal de la sauvegarde.\""""%dico
            
            cmd = "/bin/echo %(message)s | /usr/sbin/bsmtp -h %(smtp)s -f bacula@%(nom_domaine)s -s '%(numero_etab)s La sauvegarde a échoué' %(mail)s"%dico
            os.system(cmd)
        except:
            pass


bacula.set_events(BaculaEvents()) # register daemon events desired


